//_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/
// PanelSim 1.0
// Ralf Stoffels - 2015
// 
// Hiermit wird das C-Hack-Panel simuliert, so dass wir die 
// Kommunikation ausprobieren können ohne das Panel zu brauchen
//_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/


import processing.serial.*;
Serial myPort;  

int counter = 0;
int bitCounter = 1;
int loopCount = 0;
boolean recordIoData = false;
int ioCounter = 0;
int panelNumber;

int[] data;

int[][] panel = new int[][] {
  {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
  {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
  {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
  {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
  {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
  {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
  {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}
};

void setup() {
  size(1000, 340);
  background(0);
  println("Serial Interfaces found!");
  println(Serial.list());
  myPort = new Serial(this, "/dev/ttyUSB0", 115200);
  updatePanel();
}

void draw() {
   while (myPort.available() > 0) {
      int inByte = myPort.read();
      if (processByte((byte) inByte) == true) updatePanel();
   }    
}

// Write a received byte into the data array
boolean processByte(byte b) {
  int panelRow, panelColumn, value, factor;
  boolean panelComplete;

  panelComplete = false;
  if((b & 128) != 0) 
  {
    panelNumber = b & 127;
    recordIoData = true;
    clearPanel(panelNumber);
  }
  else
  {
    if(recordIoData)
    {
      panelColumn = (panelNumber) * 4 - ioCounter/3 - 1;
      println("-->"+ panelColumn);
      value=1; panelRow = 0; factor = 1;
      if (ioCounter % 3 == 0) factor = 1;
      if (ioCounter % 3 == 1) factor = 4;
      if (ioCounter % 3 == 2) factor = 2;
      for (int i=0; i<7; i++) {
        if ((b & value) > 0) panel[6-i][panelColumn] += factor;
        value *= 2;
        panelRow++;
      }
      ioCounter++;
      
      if(ioCounter >= 12) 
      {
        recordIoData = false;
        ioCounter = 0;
        panelComplete = true;
      }
    }
  }
  return (panelComplete);
}

void clearPanel(int panelID) {
  int startCol;
  
  startCol = (panelID-1)*4;
  for (int i=0; i<4; i++) 
     for (int j=0; j<7; j++) panel[j][i+startCol] = 0;   
}  


// Hier werden die LEDs gezeichnet und die Schaltflächen
void updatePanel() {
  drawBorders();
  for (int i=0; i<24; i++) 
     for (int j=0; j<7; j++) drawLED(i, j, panel[j][i], false);   
}  

//Hier wird eine LED gezeichnet an der logischen Koorinate x,y
// Hierbei kann x zwichen 0 und 23 liegen und y zwischen 0 und 6 
void drawLED(int x, int y, int rgb, boolean hilite) {
  int r=0,b=0,g=0;
  if (hilite) {
     stroke(255);
     strokeWeight(3);
  } else {
     strokeWeight(4);
     stroke(0);
     ellipse(40+x*40,40+y*40,20,20);
     stroke(150);
     strokeWeight(1);
  }
  if ((rgb & 1) > 0) r=255;
  if ((rgb & 2) > 0) g=255;
  if ((rgb & 4) > 0) b=255;
  
  if (rgb>=0) fill(r,g,b); else noFill();
  ellipse(40+x*40,40+y*40,20,20);
}

// Diese Funktion zeichgnet die 6 Rechtecke der Panels
void drawBorders() {
  fill(0);
  stroke(255);
  strokeWeight(1);
  for (int i=0; i<6; i++) rect(20+160*i,20,160,300);
}

void keyPressed() {

  if (key == 127) {
      println((int) key);
    for (int i=1; i<7; i++) clearPanel(i);
  }
  updatePanel();
}

