/*********************************************************
panelCTRL.c  CC4.0 BY-SA.NC

Controls the C-Hack Panel via serial interface
The SW accepts ASCII Strimngs or a reference to a pixel file

usage panelCtrl -s <6 ascii characters with color info>
                   Example: aBbBcWdGeTFW

      paneltest -f <file name>

**********************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <wiringSerial.h>
#include <unistd.h>

#define BAUDRATE 115200
#define DEFAULT_DEVICE "/dev/ttyAMA0"

#define TRUE 0
#define FALSE -1

/* ------------------------- Global Variables -------------------------------------*/

// This buffer holds the 3 colors per column and 12 columns in total
int buffer[7][24] = {0,2,2,2, 4,0,0,4, 1,1,1,1, 0,3,3,3, 5,0,0,5, 0,0,7,0,
                     2,0,0,0, 4,0,0,4, 1,0,0,0, 3,0,0,0, 5,0,5,0, 0,0,7,0,
                     2,0,0,0, 4,0,0,4, 1,0,0,0, 3,0,0,0, 5,5,0,0, 0,0,7,0,
                     2,0,0,0, 4,4,4,4, 1,1,1,0, 3,0,0,0, 5,0,0,0, 0,0,7,0,
                     2,0,0,0, 4,0,0,4, 1,0,0,0, 3,0,0,0, 5,5,0,0, 0,0,7,0,
                     2,0,0,0, 4,0,0,4, 1,0,0,0, 3,0,0,0, 5,0,5,0, 0,0,0,0,
                     0,2,2,2, 4,0,0,4, 1,1,1,1, 0,3,3,3, 5,0,0,5, 0,0,7,0};  

char panelText[100][12];   // Contains the text to be displayed on the panels 12 = 6 panels + 6 color codes
int fd;   // Handle for serial interface
int sequencerDelay;  // Milliseconds between two display setting fo the panels
char sequencerFileName[1000];
char serialIOName[200];
int textMode, fileMode, verbose;

/*-------------------------------- Functions ----------------------------------------------*/

// Writes the entire buffer to all panels
void writeBuffer() {
   unsigned char i;
   int x, y, factor;
   unsigned char R, G, B;
   for (i=1; i<7; i++) {
      serialPutchar (fd, i | 128);  //  write panel address
      //printf("Panel #%i\n",i);   // Only for Debug


      for (x=(i-1)*4+3; x>=(i-1)*4; x--) {
         R=0; G=0; B=0 ; factor=1;

         for (y=6; y>=0; y--) {
            if ((buffer[y][x] & 1) > 0) R = R + factor;
            if ((buffer[y][x] & 4) > 0) G = G + factor;
            if ((buffer[y][x] & 2) > 0) B = B + factor;
            factor *= 2;
         }

         serialPutchar(fd, R);  // Write the 3 bytes of one column
         serialPutchar(fd, G);
         serialPutchar(fd, B);

         //printf("%i %i  %i\n",R,G,B);   // Only for Debug
      }
   }
}

// Helptext
void printUsage() {
	fprintf(stderr, "panelCtrl CC4.0 BY-NC-SA 2015          c-hack.de\n\n");
	fprintf(stderr, "Usage: \n");
	fprintf(stderr, "File Mode: panelCtrl -d <ms> -f <filename>\n");
	fprintf(stderr, "Text Mode: panelCtrl -d <ms> -s <textstring teststring textstring>\n\n");
	fprintf(stderr, "-d  pause between two panel outputs in milliseconds\n");
	fprintf(stderr, "-v  prints the parsed data to stdout\n");
	fprintf(stderr, "-i  specifies serial interface. Default: \\dev\\ttyAMA0\n");
	fprintf(stderr, "-f  filename which contains pixel maps\n");
	fprintf(stderr, "-s  text output. Format: <character><color>...\n");
	fprintf(stderr, "    example: -d 1000 -s tBeBsBtB tReRsRtR\n             writes the word test and alternates between blue and red test \n");
	fprintf(stderr, "             Colors: B=blue, G=green, R=red, T=turquoise, P=purple, Y=yellow, W=white\n");
	exit(1);
} 

// Sequences the textstrings as defined in the *text[] parameter
void textSequencer(int seqDelay, char text[100][12]) {
	fprintf(stderr, "Not implemented yet - sorry!\n");
};

// Sequences the file content as specified by the filename parameter
void fileSequencer(int seqDelay, char *filename) {
   FILE *pFile;
   int valid, code;
   int c, i;
   int x,y;
   int fileRowCount;
   char delayStr[8];
   
   pFile = fopen (filename,"r");
   if (pFile == NULL) {
	    fprintf(stderr,"ERROR opening file: %s\n",filename);
        exit(1);
   }

   while (1==1) {
       valid = TRUE;
       x=0;
       y=0;
       fileRowCount = 1;
	   do {
		  c = fgetc(pFile);
	      if (c == '#') valid = FALSE;  // everthing after a '#' gets ignored
	      if (c == '$') {   // new delay value
			 i=0;
			 do {  // max 5 digits
				 c = fgetc(pFile);
				 delayStr[i++] = c;  
			 }  while ((c>='0')&&(c<='9')&&(i<5));
			 delayStr[i] = '\0';  // String-End 
			 seqDelay = atoi(delayStr);
			 if (verbose == TRUE) printf("Delay = %i\n",seqDelay);
		  }
			  
	      if (c == '\n') {
			  valid = TRUE;  // until line end
			  fileRowCount++;
			  if (verbose == TRUE) printf("\n");
		  }
	      if (valid == TRUE) {
			  switch(c) {
		         case '.': code=0; break;
		         case 'R': code=1; break;
		         case 'G': code=2; break;
		         case 'B': code=4; break;
		         case 'T': code=6; break;
		         case 'Y': code=3; break;
		         case 'P': code=5; break;
		         case 'W': code=7; break;
		         default : code=-1; break;
			  }
			  if (code != -1) {
				  if (verbose == TRUE) printf("%c",c);
				  if ((y < 7) && (x<24)) { 
					  buffer[y][x] = code;
				  } else {
					  fprintf(stderr, "FILE SYNTAX ERROR in line %i  ",fileRowCount); 
					  if (verbose == TRUE) fprintf(stderr, "x = %i  y = %i\n",x,y); 
					  exit(1);
				  }
				  x++;
				  if (x>23) {
					  x=0;   // Back to line begin
					  y++;
				  }
			  if (y > 6) {   // One panel complete
				  y=0;  // Back to row 1
				  writeBuffer();
				  delay(seqDelay);
				  if (verbose == TRUE) printf("\n------------\n");
			  }
			}
		  }  
	   } while (c != EOF);
	   rewind(pFile);
	   if (verbose == TRUE) printf("\n#####################\n\n");
   }   
   fclose (pFile);
}	

/*-------------------------------- Main: Command Line Interface ------------------------------------*/
main(int argc, char *argv[]) {
   int i,j,k;
   
   textMode = FALSE;
   fileMode = FALSE;
   verbose = FALSE;
   strcpy(serialIOName, DEFAULT_DEVICE); 
   
   // Command line processing
   if (argc < 2) printUsage();
   for (i=1; i<argc; i++) {
	   if (strcmp(argv[i],"-d") == 0) {
		   sequencerDelay = atoi(argv[i+1]);
	   } 
	   
	   if (strcmp(argv[i],"-v") == 0) {
		   verbose = TRUE;
	   } 
	   
	   if (strcmp(argv[i],"-s") == 0) {
		   textMode = TRUE;  fileMode = FALSE;   // Cannot be both enabled
		   k = 0;
		   for (j=i+1; j<argc; j++) {
			   if (k<100) strncpy(panelText[k++],argv[j],12);
			   printf("%s ",panelText[k-1]);
		   }
		   printf("\n");
	   }
	   
	   if (strcmp(argv[i],"-f") == 0) {
		   fileMode = TRUE; textMode = FALSE;    // Cannot be both enabled
		   strncpy(sequencerFileName, argv[i+1], 1000);
    	   printf("Filename = %s\n",sequencerFileName);
	   }

	   if (strcmp(argv[i],"-i") == 0) {
		   strncpy(serialIOName, argv[i+1], 200);
	   }
   }
     
   printf("Delay = %ims\n",sequencerDelay);
   printf("Serial = %s\n",serialIOName);   

   // Initialize the serial interface
   fd = serialOpen (serialIOName, BAUDRATE);
   if (fd<0) {
	fprintf(stderr, "ERROR: %s not found\n",serialIOName);
        exit(1);
   }
   
   if (fileMode == TRUE) fileSequencer(sequencerDelay, sequencerFileName);
   if (textMode == TRUE) textSequencer(sequencerDelay, panelText); 

   serialClose(fd); 
} 


